@@ -13,6 +13,23 @@ import click
 PathLike = Union[Path, str]
 
 
+SKIPPABLE_MAGIC_CODES = [
+    # non-python languages
+    'bash',
+    'html',
+    'javascript',
+    'js',
+    'latex',
+    'markdown',
+    'perl',
+    'ruby',
+    'sh',
+    'svg',
+    # extra functionality
+    'writefile',
+]
+
+
 def format_file(
     notebook_path: PathLike,
     mode: black.FileMode,
@@ -50,13 +67,19 @@ def format_file(
         # format code
         orig_source = cell['source']
 
-        # black expects empty line at end of file
+        # check whether we should really process cell
+        if any(orig_source.startswith(f'%%{magic}') for magic in SKIPPABLE_MAGIC_CODES):
+            continue
+
+        # black expects empty line at end of non-empty file
         # for notebook cells, this does not make sense
-        orig_source += '\n'
+        if len(orig_source) > 0:
+            orig_source += '\n'
 
         # Jupyter cell magic can mess up black
         # TODO: this is a bad hack
         orig_source = re.sub('^%', '#%#jupylint#', orig_source, flags=re.M)
+        orig_source = re.sub('^!', '#!#jupylint#', orig_source, flags=re.M)
 
         try:
             fmted_source = black.format_str(orig_source, mode=mode)
@@ -66,10 +89,13 @@ def format_file(
                 cells_errored += 1
                 continue
             else:
-                raise e
+                raise RuntimeError(
+                    f'[{notebook_path}] Error while formatting cell {i}: {e}'
+                )
 
         if orig_source != fmted_source:
             fmted_source = re.sub('^#%#jupylint#', '%', fmted_source, flags=re.M)
+            fmted_source = re.sub('^#!#jupylint#', '!', fmted_source, flags=re.M)
 
             header = f'{notebook_path} - Cell {i} '
 
@@ -174,7 +200,7 @@ def get_notebooks_in_dir(path, exclude_regex):
     '--exclude',
     type=str,
     metavar='PATTERN',
-    default=r'(.git|.ipynb_checkpoints|build|dist)',
+    default=r'(/.git/|/.ipynb_checkpoints/|/build/|/dist/)',
     help=(
         'Regular expression to match paths which should be exluded when searching directories.'
     ),
